class TButtonArea {
    inherit iwidgets::Scrolledframe

    variable CRowList
    variable CComArray

    variable CVarM
    variable CInput
    method connect {aInput aVarM} {
	set CVarM  $aVarM
	set CInput $aInput
    }

    proc widNameIsOK {aName} {
	return [regexp -- {^[0-9a-zA-Z]+$} $aName]
    }
    constructor { args } {
	configure -borderwidth 2 
	configure -relief ridge 
	configure -vscrollmode dynamic 
	configure -hscrollmode dynamic 
	set CRowList [list]
	eval itk_initialize $args
    }
    method clear {} {
	foreach wid $CRowList {
	    destroy $wid
	}
    }
    method rowExists {aRowName} { return [info exists itk_component(row,$aRowName)] }
    method addButtonRow { aRowName } {
	#rows start with row
	if {![widNameIsOK $aRowName]} { return 0 }
	set wid [childsite].row$aRowName
	if {[winfo exists $wid]}      { return 0 }
	itk_component add row,$aRowName { frame $wid -borderwidth 2 -relief ridge } {keep -background}
	pack $itk_component(row,$aRowName) -side top -expand 0 -fill x -ipadx 0 -ipady 0 -anchor nw
	lappend CRowList $itk_component(row,$aRowName)
	return 1
    }
    method remButtonRow { aRowName } {
	if {![rowExists $aRowName]} { return 0 }
	destroy $itk_component(row,$aRowName)
	return 1
    }
    method addButton {aRowName aButtonName aText aCom} {
	# buttons start with but
	if {![rowExists $aRowName]}      { return 0 }
	if {![widNameIsOK $aButtonName]} { return 0 }
	set wid $itk_component(row,$aRowName).but$aButtonName
	if {[winfo exists $wid]}         { return 0 }	
	set CComArray($aRowName,$aButtonName) $aCom
	itk_component   add but,$aRowName,$aButtonName { 
	    button $wid -text $aText -command "[code $this invokeCom $aRowName $aButtonName]" -font small 
	} {keep -background}
	pack $itk_component(but,$aRowName,$aButtonName) -side left -expand 0 -fill y -ipadx 0 -ipady 0 -anchor nw
	return 1
    }
    method invokeCom {aRowName aButtonName} {
	$CInput takeCommand $CComArray($aRowName,$aButtonName)
    }
    method remButton {aRowName aButtonName} {
	if {![info exists itk_component(but,$aRowName,$aButtonName)]} { return 0 }
	destroy $itk_component(but,$aRowName,$aButtonName)
	return 1
    }
    method addLabel {aRowName aLabelName aText aWidth} {
	# labels start with lab
	if {![rowExists $aRowName]}        { return 0 }
	if {![widNameIsOK $aLabelName]}    { return 0 }
	if {![regexp -- {[0-9]+} $aWidth]} { return 0 }
	set wid $itk_component(row,$aRowName).lab$aLabelName
	if {[winfo exists $wid]}           { return 0 }
	itk_component   add lab,$aRowName,$aLabelName { 
	    label $wid -text $aText -width $aWidth -font small -justify left -anchor w \
		-borderwidth 2 
	} {keep -background}
	pack $itk_component(lab,$aRowName,$aLabelName) -side left -expand 0 -fill y -ipadx 0 -ipady 0 -anchor nw
	return 1
    }
    method remLabel {aRowName aLabelName} {
	if {![info exists itk_component(lab,$aRowName,$aLabelName)]} { return 0 }
	destroy $itk_component(lab,$aRowName,$aLabelName)
	return 1
    }
    method setLabelText {aRowName aLabelName aText} {
	if {![info exists itk_component(lab,$aRowName,$aLabelName)]} { return 0 }
	$itk_component(lab,$aRowName,$aLabelName) configure -text $aText
	return 1
    }
    method addVarEntry {aRowName aName aVarName aWidth } {
	# varentries start with ven
	if {![rowExists $aRowName]}        { return 0 }
	if {![widNameIsOK $aName]}         { return 0 }
	if {![regexp -- {[0-9]+} $aWidth]} { return 0 }
	set wid $itk_component(row,$aRowName).ven$aName
	if {[winfo exists $wid]}           { return 0 }	

	$CVarM createIfNeeded $aVarName

	itk_component   add ven,$aRowName,$aName {
	    entry $wid -font small -textvariable [$CVarM getVarName $aVarName] -width $aWidth \
		-borderwidth 2 -justify left
	} {keep -background}
	pack $itk_component(ven,$aRowName,$aName) -side left -expand 0 -fill y -ipadx 0 -ipady 0 -anchor nw
	return 1
    }
    method remVarEntry {aRowName aName} {
	if {![info exists itk_component(ven,$aRowName,$aName)]} { return 0 }
	destroy $itk_component(ven,$aRowName,$aName)
	return 1
    }
    method addVarLabel {aRowName aName aVarName aWidth } {
	# varlabels start with vla
	if {![rowExists $aRowName]}        { return 0 }
	if {![widNameIsOK $aName]}         { return 0 }
	if {![regexp -- {[0-9]+} $aWidth]} { return 0 }
	set wid $itk_component(row,$aRowName).vla$aName
	if {[winfo exists $wid]}           { return 0 }	

	$CVarM createIfNeeded $aVarName

	itk_component   add vla,$aRowName,$aName {
	    label $wid -font small -textvariable [$CVarM getVarName $aVarName] -width $aWidth \
		-relief ridge -borderwidth 2 -justify left -anchor w
	} {keep -background}
	pack $itk_component(vla,$aRowName,$aName) -side left -expand 0 -fill y -ipadx 0 -ipady 0 -anchor nw
	return 1
    }
    method remVarLabel {aRowName aName} {
	if {![info exists itk_component(vla,$aRowName,$aName)]} { return 0 }
	destroy $itk_component(vla,$aRowName,$aName)
	return 1
    }

    method addVarChooser {aRowName aName aVarName aWidth } {
	# varchooser start with vch
	if {![rowExists $aRowName]}        { return 0 }
	if {![widNameIsOK $aName]}         { return 0 }
	if {![regexp -- {[0-9]+} $aWidth]} { return 0 }
	set wid $itk_component(row,$aRowName).vch$aName
	if {[winfo exists $wid]}           { return 0 }	

	$CVarM createIfNeeded $aVarName

	itk_component   add vch,$aRowName,$aName {
	    iwidgets::combobox $wid -textfont small -width $aWidth -textvariable [$CVarM getVarName $aVarName] \
		-unique true
	} {keep -background}
	pack $itk_component(vch,$aRowName,$aName) -side left -expand 0 -fill y -ipadx 0 -ipady 0 -anchor nw
	return 1
    }
    method remVarChooser {aRowName aName} {
	if {![info exists itk_component(vch,$aRowName,$aName)]} { return 0 }
	destroy $itk_component(vch,$aRowName,$aName)
	return 1
    }
    method addVarChooserItem {aRowName aName aItem} {
	if {![info exists itk_component(vch,$aRowName,$aName)]} { return 0 }
	$itk_component(vch,$aRowName,$aName) insert list end $aItem
	return 1
    }
    method remVarChooserItem {aRowName aName aIndex} {
	if {![info exists itk_component(vch,$aRowName,$aName)]} { return 0 }
	$itk_component(vch,$aRowName,$aName) delete list $aItem $aItem
	return 1
    }
}
