class TMouseButtonConfigurer {
    inherit iwidgets::Texteddialog
    
    variable CFilM

    variable CB2
    variable CB3
    variable CM2
    variable CM3
    variable CB2r
    variable CB3r
    variable CM2r
    variable CM3r

    constructor {aFilM args} {
	set CFilM   $aFilM
	loadMouserc

	configure -title "SMM++ Mouse Button Configuration"
	configure -modality application
	configure -text "Here you can set how 'middle-' and 'right-mousebutton-click' events are invoked.\
                       \nSo if you dont have a three button mouse, you can redirect the events and\
                         still access all SMM's functionality.\
                         If you accept, your changes will be saved, too."
	configure -aspect 333
	buttonconfigure OK     -text "Accept" -command "[code $this save]"
	buttonconfigure Cancel -text "Cancel" -command "[code $this canc]"

	event add <<Button-2>> <Button-2>
	event add <<Double-2>> <Double-2>
	event add <<Button-3>> <Button-3>
	event add <<Double-3>> <Double-3>

	# -- configurer
	set wid [$this childsite]
	itk_component add middlebutton {
	    TButtonActionChooser $wid.mb "Middle Mouse Button" $this $CB2r $CM2r
	} { keep -background}

	itk_component add sep { iwidgets::line $wid.sep } { keep -background }

	itk_component add rightbutton {
	    TButtonActionChooser $wid.rb "Right Mouse Button " $this $CB3r $CM3r
	} { keep -background}
 	pack $itk_component(middlebutton) $itk_component(sep) $itk_component(rightbutton) \
	    -side left -fill both -expand true -ipadx 0 -ipady 0 -padx 0 -pady 0

	eval itk_initialize $args	
	setRealSelectors
	adjustBindings

    }
    method adjustBindings {} {
	event delete <<Button-2>>
	event delete <<Button-3>>
	event delete <<Double-2>>
	event delete <<Double-3>>

	if {![string compare $CM2r "None"]} {
	    set m2 ""
	} else {
	    set m2 $CM2r
	}
	if {![string compare $CM3r "None"]} {
	    set m3 ""
	} else {
	    set m3 $CM3r
	}

	event add <<Button-2>> "<$m2$CB2r>"
	event add <<Button-3>> "<$m3$CB3r>"
	event add <<Double-2>> "<$m2\Double-$CB2r>"
	event add <<Double-3>> "<$m3\Double-$CB3r>"
	
	
    }
    
    method setRealSelectors {} {
	setSelectors $CM2r $CB2r $CM3r $CB3r
    }

    method loadMouserc {} {
	set f [$CFilM getMouserc]	
	if {[file exists $f]} {
	    set fid [open $f "r"]
	    gets $fid version

	    switch -- $version {
		1 {
		    gets $fid CB2
		    gets $fid CM2
		    gets $fid CB3
		    gets $fid CM3
		}
		default {
		    error "Unknown version '$version' for mouserc!"
		}
	    }	    
	    close $fid

	} else {
	    set CB2 Button-2
	    set CM2 None
	    set CB3 Button-3
	    set CM3 None
	} 
	set CB2r $CB2 
	set CM2r $CM2 
	set CB3r $CB3 
	set CM3r $CM3 
    }

    method feedback {} {
	set CB2 [$itk_component(middlebutton) getButton]
	set CM2 [$itk_component(middlebutton) getModifier]
	set CB3 [$itk_component(rightbutton) getButton]
	set CM3 [$itk_component(rightbutton) getModifier]
	setSelectors $CM2 $CB2 $CM3 $CB3
    }

    method setSelectors {aCM2 aCB2 aCM3 aCB3} {
	$itk_component(middlebutton) turnOffFeedback
	$itk_component(rightbutton) turnOffFeedback

	$itk_component(middlebutton) enableall
	$itk_component(rightbutton) enableall

	$itk_component(middlebutton) select $aCB2 $aCM2
	$itk_component(rightbutton) select $aCB3 $aCM3

	if {![string compare $aCB2 "Button-1"]} {
	    $itk_component(middlebutton) disable modsel 0
	} 
	if {![string compare $aCM2 "None"]} {
	    $itk_component(middlebutton) disable buttonsel 0
	} 

	if {![string compare $aCB3 "Button-1"]} {
	    $itk_component(rightbutton) disable modsel 0
	} 
	if {![string compare $aCM3 "None"]} {
	    $itk_component(rightbutton) disable buttonsel 0
	}

	if {![string compare $aCB2 $aCB3]} {
	    $itk_component(middlebutton) disable modsel $aCM3
	    $itk_component(rightbutton) disable modsel $aCM2
	}
	if {![string compare $aCM2 $aCM3]} {
	    $itk_component(middlebutton) disable buttonsel $aCB3
	    $itk_component(rightbutton) disable buttonsel $aCB2
	}

	$itk_component(middlebutton) turnOnFeedback
	$itk_component(rightbutton) turnOnFeedback

    }

    method save {} {
	set f   [$CFilM getMouserc]	

	set fid [open $f "w"]
	puts $fid 1 ;# version
	puts $fid $CB2
	puts $fid $CM2
	puts $fid $CB3
	puts $fid $CM3
	close $fid

	set CB2r $CB2 
	set CM2r $CM2 
	set CB3r $CB3 
	set CM3r $CM3 	

	adjustBindings
	deactivate
    }

    method canc {} { 
	setRealSelectors
	deactivate 
    }

}

class TButtonActionChooser {
    inherit itk::Widget

    variable CParent

    constructor {tit parent s1 s2 args} {
	set CParent $parent

	itk_component add title {
	    label $itk_interior.tit -text $tit -font defBold
	} {keep -background}
 	pack $itk_component(title) -side top -fill both -expand true 

# 	itk_component add sep { iwidgets::line $itk_interior.sep } { keep -background }
#  	pack $itk_component(sep) -side top -fill both -expand true 
	
	itk_component add buttonsel {
	    iwidgets::radiobox $itk_interior.bs -labeltext "Mouse Button" -labelfont def
	} {keep -background}
	
	$itk_component(buttonsel) add Button-1 -text "Button 1 (Left)"    -font def
	$itk_component(buttonsel) add Button-2 -text "Button 2 (Middle)"  -font def
	$itk_component(buttonsel) add Button-3 -text "Button 3 (Right)"   -font def
 	pack $itk_component(buttonsel) -side top -fill both -expand true -ipadx 0 -ipady 0 -padx 0 -pady 0

	itk_component add modsel {
	    iwidgets::radiobox $itk_interior.ms -labeltext "Modifier" -labelfont def
	} {keep -background}
	
	$itk_component(modsel) add "None"       -text "None"      -font def
	$itk_component(modsel) add "Shift-"     -text "Shift"     -font def
	$itk_component(modsel) add "Control-"   -text "Control"   -font def
	$itk_component(modsel) add "Alt-"       -text "Alternate" -font def
 	pack $itk_component(modsel) -side top -fill both -expand true -ipadx 0 -ipady 0 -padx 0 -pady 0

	$itk_component(buttonsel) select $s1
	$itk_component(modsel)    select $s2
	turnOnFeedback
	eval itk_initialize $args	
    }
    method turnOnFeedback {} {
	$itk_component(buttonsel) configure -command [code $CParent feedback]
	$itk_component(modsel)    configure -command [code $CParent feedback]
    }
    method turnOffFeedback {} {
	$itk_component(buttonsel) configure -command {}
	$itk_component(modsel)    configure -command {}
    }
    method getButton {} {
	return [$itk_component(buttonsel) get]
    }
    method getModifier {} {
	return [$itk_component(modsel) get]
    }
    method select {s1 s2} {
	$itk_component(buttonsel) select $s1
	$itk_component(modsel)    select $s2
    }
    method disable {b i} {
	$itk_component($b) buttonconfigure $i -state disabled
    }
    method enable {b i} {
	$itk_component($b) buttonconfigure $i -state normal
    }
    method enableall {} {
	$itk_component(buttonsel) buttonconfigure 0 -state normal
	$itk_component(buttonsel) buttonconfigure 1 -state normal
	$itk_component(buttonsel) buttonconfigure 2 -state normal
	$itk_component(modsel) buttonconfigure 0 -state normal
	$itk_component(modsel) buttonconfigure 1 -state normal
	$itk_component(modsel) buttonconfigure 2 -state normal
	$itk_component(modsel) buttonconfigure 3 -state normal
    }

}
